/****************************************************************************************
    File name:	tcpc2.c
 
    Description:
	This program demo 
	how to use socket lib coding TCP Clinet program on UC7000/DA-66x series.
	
	The Hardware connection is following:
	Terminal---(RS232)[UC7000/DA-66x series]--- ~~LAN~~ ---PC Host Example	(Windows platform)
	
	This program connect to remote host with specified IP address and TCP port,
	as TCP client.And exchange data between serial port and Ethernet.

	
    Usage:
    	1. Make downloading file for UC7000/DA-66x series 
    	    1.1 Create executable file from source code(*.c) 
    	    	1.1.1 Modify tcpc2.c: 
    	    		change "DEST_IP" following the PC Host's IP address.
    	    		change "TCPPORT" if needed.	
    	    	1.1.2 Compiler and link to make tcpc2-release (no debug message) or tcpc2-debug (with debug message)
	2. Download executable file to UC7000/DA-66x series 
     	    2.1 Using ftp download tcpc2-release or tcpc2-debug to a UC7000/DA-66x series.
    	3. Execute TCP Server example for PC.[tcpserver.exe or frmServer.exe]. 
    	   input the TCP Port number equal to "TCPPORT"
     	4. Run tcpc2-release or tcpc2-debug on UC7000/DA-66x series. 

    History:    
    	Version		Author		Date		Comment
        1.0		Victor Yu.	01-14-2004      Wrote it.
****************************************************************************************/
#include        <stdio.h>
#include        <stdlib.h>
#include        <sys/types.h>
#include        <sys/socket.h>
#include	<sys/fcntl.h>
#include	<termios.h>
#include        <netinet/in.h>
#include        <errno.h>

#define SPORT		"/dev/ttyM0"
#define TCPPORT		4000
#define DEST_IP		"192.168.4.140"

#define BUFFER_LEN	1024

char	sbuf[BUFFER_LEN];

int main()
{
	int			sfd, afd;
	struct	sockaddr_in	des;
	int    			size,ret,len;
	struct termios		tty;
	
	printf("start\n");
	/* open serial port */
	afd = open(SPORT, O_RDWR|O_NDELAY);	// open port for nonblock mode
	if ( afd < 0) {
		printf("open  sync port [%s] fail [%d] !", SPORT, errno);
		return 0;
	}

	/* set serial port baud rate */
	tty.c_cflag = CREAD | CLOCAL | B38400 | CS8;
	tty.c_oflag = 0;
	tty.c_lflag = 0;
	tty.c_iflag = 0;
	tty.c_cc[VMIN] = 0;
	tty.c_cc[VTIME] = 0;
	tcsetattr(afd, TCSANOW, &tty);					

	/* create socket */
	if ( (sfd=socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0 ){
		close(afd);
		printf("socket open fail [%d] !", errno);
	   	return 0;
	}

	size = sizeof(des);
	des.sin_addr.s_addr = inet_addr(DEST_IP);
	des.sin_family = AF_INET;
	des.sin_port = htons(TCPPORT);	
	printf("connect...");
	if ( connect(sfd, (struct sockaddr *)&des, size) == 0 ) {
		printf("OK.\n");
	} else {
		close(afd);
		close(sfd);
		printf("fail [%d] !\n", errno);
		return 0;
	}
	
	/* set socket to nonblock */
	ret = fcntl(sfd, F_GETFL);
	ret |= O_NONBLOCK;
	if ( fcntl(sfd, F_SETFL, ret) < 0 ) {
		printf("set socket to nonblock fail [%d] !\n", errno);
		goto fail;
	}

	while(1){
		/* read data from serial port */
	    	if ( (len=read(afd, sbuf, BUFFER_LEN)) < 0 ) {
			printf("async port read data fail [%d] !\n", errno);
			goto fail;
		} else if(len > 0) {
		    	/* send data to LAN */
		    	if ( send(sfd, sbuf, len, 0) < 0 ) {
				printf("socket send data fail [%d] !\n", errno);
				goto fail;
		    	}
		}

		/* receive data form LAN */
		ret = recv(sfd, sbuf, BUFFER_LEN, 0);
		if ( ret == 0 ) 
  			break;
		else if ( ret < 0 ) {
			if ( errno == EAGAIN )
				continue;
			if ( errno == ENOTCONN )
				break;
			printf("receive from socket fail [%d] !\n", errno);
			goto fail;
		}else{
			/* write data to serial port */
		    	if ( (len=write(afd, sbuf, ret)) < 0 ) {
				printf("async port write data fail [%d] !\n", errno);
				goto fail;
		    	} 
		}
	}

	printf("disconnect and exit\n");
	
fail:
	close(sfd);	
	close(afd);

	return 0;
}
